#include "base.h"


#include <limits>
#include <cmath>
#include <stdlib.h>
#include <time.h>

using namespace std;
using namespace control;

void integrate::init(){
    out = param_initial_value;
}

void integrate::tick(){
    out += 0.01 * in;
    if(out > param_max){
        out = param_max;
    }
    if(out < param_min){
        out = param_min;
    }
    if(in_reset){
        init();
    }
}


void derive::init(){
    last_in = in;
}

void derive::tick(){
    out = 100 * (in - last_in);
    last_in = in;
}


void pt1::init(){
    out = param_initial_value;
}

void pt1::tick(){
    out = out + 0.01/param_t * (in - out);
}


void pt2::init(){
    double T = 1.0/2.0/M_PI*param_t;
    T_inv    = 1.0/T;
    TT_inv   = 1.0/(T*T);
    out      = param_initial_value;
    out_derivative = param_initial_value_derivative;
}

void pt2::tick(){
    out_derivative = out_derivative + 0.01 * (in*TT_inv - out*TT_inv - out_derivative*2*param_d*T_inv);
    out += 0.01 * out_derivative;
}


void limit::tick(){
    out = in;
    if(out > param_max){
        out = param_max;
    }
    if(out < param_min){
        out = param_min;
    }
}


void noise::init(){
    srand( time(NULL) );
}

void noise::tick(){
    out = (double) rand()/RAND_MAX*2.0 - 1.0;
}


void edge_detect::init(){
    last_in = in;
}

void edge_detect::tick(){
    out = false;
    if(last_in == false && in && param_rising){
        out  = true;
    }
    if(last_in && in == false && param_falling){
        out  = true;
    }
    last_in = in;
}



void avg_filter::init(){
    data = new double[param_size];
    for(int i = 0; i < param_size; ++i){
        data[i] = param_initial_value/param_size;
    }
    pos = 0;
    out = param_initial_value;
}

void avg_filter::tick(){
    out -= data[pos];
    data[pos] = in/param_size;
    out += data[pos];
    ++pos;
    if(pos >= param_size){
        pos = 0;
    }
}

void avg_filter::final(){
    delete[] data;
}


void dt::init(){
    last_call = abmt::time::now() - abmt::time::ns(param_first_dt * 1000.0*1000.0*1000.0);
}

void dt::tick(){
    auto now   = abmt::time::now();
    auto delta = now - last_call;
    out_dt     = delta.sec();
    last_call  = now;
}

void min::init(){
    out =std::numeric_limits<double>::max();
}

void min::tick(){
    if(in < out){
        out = in;
    }
    if(in_reset){
        out = in;
    }
}

void max::init(){
    out = std::numeric_limits<double>::lowest();
}

void max::tick(){
    if(in > out){
        out = in;
    }
    if(in_reset){
        out = in;
    }
}

void interpolate::tick(){
    if(in > param_in_max){
        in = param_in_max;
    }
    if(in < param_in_min){
        in = param_in_min;
    }
    double out_delta = param_out_max - param_out_min;
    double in_delta = param_in_max - param_in_min;
    out = param_out_min + (in-param_in_min)/(in_delta) * out_delta;
}