#ifndef CONTROL_BASE_H
#define CONTROL_BASE_H CONTROL_BASE_H

#include <cstddef>
#include <abmt/time.h>

namespace control{
    
//@node: auto
//@raster: 10ms
struct integrate{
    double out;
    double in = 0;
    bool   in_reset = false;
    double param_initial_value = 0;
    double param_min = -1000000000000;
    double param_max =  1000000000000;
    void init();
    void tick();
};

//@node: auto
//@raster: 10ms
struct derive{
    double out;
    double in;
    double last_in;
    void init();
    void tick();
};

//@node: auto
//@raster: 10ms
struct pt1{
    double out;
    double in = 0;
    double param_t = 1;
    double param_initial_value = 0;
    
    void init();
    void tick();
};

//@node: auto
//@raster: 10ms
struct pt2{
    double out;
    double out_derivative;
    double in = 0;
    double param_t  = 1;
    double param_d  = 1;
    double param_initial_value = 0;
    double param_initial_value_derivative = 0;
    double T_inv;
    double TT_inv;
    
    
    void init();
    void tick();
};

//@node: auto
struct limit{
    double out;
    double in = 0;

    double param_min = 0;
    double param_max = 1;
    
    void tick();
};

//@node: auto
struct noise{
    double out;
    void init();
    void tick();
};


//@node: auto 
struct edge_detect{
    bool out;
    bool in = 0;
    bool param_rising  = true;
    bool param_falling = false;
    
    bool last_in = 0;
    void init();
    void tick();
};

//@node: auto
struct avg_filter{
    double out;
    double in = 0;
    size_t param_size = 10;
    double param_initial_value = 0;
    
    size_t pos = 0;
    
    double* data;
    
    void init();
    void tick();
    void final();
};

//@node: auto
struct dt{
    double out_dt;
    double param_first_dt = 0.01;
    abmt::time last_call; 
    
    void init();
    void tick();
};


//@node: auto
//@raster: 10ms
struct min{
    double out;
    double in = 0;
    bool   in_reset = false;
    void init();
    void tick();
};

//@node: auto
//@raster: 10ms
struct max{
    double out;
    double in = 0;
    bool   in_reset = false;
    void init();
    void tick();
};

//@node: auto
//@raster: 10ms
struct interpolate{
    double out;
    double in = 0;
    double param_in_min = 0;
    double param_in_max = 1;
    double param_out_min = 0;
    double param_out_max = 1;
    void tick();
};


} // namespace control

#endif // CONTROL_BASE_H