#include "gpio.h"

#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/gpio.h>
#include <cstring>
#include <iostream>

#include <abmt/os.h>

using namespace std;
using namespace io;

struct gpio_data{
    gpiohandle_request req  = {};
	gpiohandle_data    data = {};
};

void gpio_output::init(){
    gpio_data* gpio = new gpio_data;
    data.set(gpio);
    int fd = open(param_device.c_str(), 0);
    if (fd < 0) {
		abmt::os::die("unable to open " + param_device + "["+to_string(param_pin)+"]: "  + std::strerror(errno));
	}
	gpio->req.lineoffsets[0] = param_pin;
	gpio->req.flags = GPIOHANDLE_REQUEST_OUTPUT;
	strcpy(gpio->req.consumer_label, "abmt");
	gpio->req.lines  = 1;
	
	int ret = ioctl(fd, GPIO_GET_LINEHANDLE_IOCTL, &(gpio->req));
	if (ret == -1) {
	    close(fd);
		abmt::os::die("error ioctl GPIO_GET_LINEHANDLE_IOCTL for " + param_device + "["+to_string(param_pin)+"]: " + std::strerror(errno));
	}
	if (close(fd) == -1){
	    abmt::os::die("unable to close " + param_device + ": "  + std::strerror(errno));
	}

}

void gpio_output::tick(){
    auto& gpio = data.get<gpio_data>();
    gpio.data.values[0] = in;
	int ret = ioctl(gpio.req.fd, GPIOHANDLE_SET_LINE_VALUES_IOCTL, &gpio.data);
	if (ret == -1) {
		abmt::os::die("error ioctl GPIOHANDLE_SET_LINE_VALUES_IOCTL for " + param_device + "["+to_string(param_pin)+"]: "  + std::strerror(errno));
	}
}

void gpio_output::final(){
    auto& gpio = data.get<gpio_data>();
    if(gpio.req.fd > 0){
        close(gpio.req.fd);
        gpio.req.fd = -1;
    }
}

gpio_output::~gpio_output(){
    final();
}

void gpio_input::init(){
    gpio_data* gpio = new gpio_data;
    data.set(gpio);
    int fd = open(param_device.c_str(), 0);
    if (fd < 0) {
		abmt::os::die("unable to open " + param_device + "["+to_string(param_pin)+"]: "  + std::strerror(errno));
	}
	gpio->req.lineoffsets[0] = param_pin;
	gpio->req.flags = GPIOHANDLE_REQUEST_INPUT;
	strcpy(gpio->req.consumer_label, "abmt");
	gpio->req.lines  = 1;
	
	int ret = ioctl(fd, GPIO_GET_LINEHANDLE_IOCTL, &(gpio->req));
	if (ret == -1) {
	    close(fd);
		abmt::os::die("error ioctl GPIO_GET_LINEHANDLE_IOCTL for " + param_device + "["+to_string(param_pin)+"]: " + std::strerror(errno));
	}
	if (close(fd) == -1){
	    abmt::os::die("unable to close " + param_device + ": "  + std::strerror(errno));
	}

}

void gpio_input::tick(){
    auto& gpio = data.get<gpio_data>();
	int ret = ioctl(gpio.req.fd, GPIOHANDLE_GET_LINE_VALUES_IOCTL, &gpio.data);
	if (ret == -1) {
		abmt::os::die("error ioctl GPIOHANDLE_SET_LINE_VALUES_IOCTL for " + param_device + "["+to_string(param_pin)+"]: "  + std::strerror(errno));
	}
	out = gpio.data.values[0];
}

void gpio_input::final(){
    auto& gpio = data.get<gpio_data>();
    if(gpio.req.fd > 0){
        close(gpio.req.fd);
        gpio.req.fd = -1;
    }
}

gpio_input::~gpio_input(){
    final();
}

void gpio_soft_pwm::tick(){
    static size_t pos = 0;
    
    if(in > 1){
        in = 1;
    }
    
    if(in < 0){
        in = 0;
    }
    
    size_t limit = in * param_steps + 0.5;
    
    if(pos >= limit){
        out = false;
    }else{
        out = true;
    }
    
    ++pos;
    if(pos >= param_steps){
        pos = 0;
    }
}


