#include "i2c.h"
#include <fcntl.h>
#include <linux/i2c.h>
#include <linux/i2c-dev.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <cstring>
#include <iostream>

#include <abmt/os.h>

using namespace std;
using namespace io;

i2c_bus_linux::i2c_bus_linux(std::string device){
    fd = open(device.c_str(), O_RDWR);
    if(fd < 0){
           abmt::os::die("unable to open " + device + ": "  + std::strerror(errno));
    }

}

bool i2c_bus_linux::write_read(uint16_t addr, void* write, uint16_t w_len, void* read, uint16_t r_len){
    auto lock = mutex.get_scope_lock(); // in case someone illegally uses the same bus in different rasters
    int res;
    if(r_len == 0 && w_len == 0){
            return true;
    }
    i2c_msg messages[] = {
            { addr, 0, w_len, (uint8_t*) write },
            { addr, I2C_M_RD | I2C_M_NOSTART, r_len, (uint8_t*) read },
    };
    i2c_rdwr_ioctl_data ioctl_data = { messages, 2 };
    if(r_len == 0 && w_len != 0){
            ioctl_data = { &messages[0], 1 };       
    }       
    if(r_len != 0 && w_len == 0){
            messages[1].flags = I2C_M_RD;
            ioctl_data = { &messages[1], 1 };       
    }
    res = ioctl(fd, I2C_RDWR, &ioctl_data);
    if (res < 0){
        if(unknown_ids.find(addr) == unknown_ids.end()){
            unknown_ids.insert(addr);
            std::cout << "i2c error. Address '0x" << (hex) << addr << "' is not responding" << std::endl;
        }
        return false;
        //abmt::os::die("read addr " + to_string(addr) + " on device" + device + ": "  + std::strerror(errno));
    }
    return true;
}

i2c_bus_linux::~i2c_bus_linux(){
    if(fd > 0){
            close(fd);
    }
}

void i2c::init(){
    out_i2c_bus.reset(new i2c_bus_linux(param_device));
}
