#include "net.h"
#include <iostream>

using namespace std;
using namespace io;

void udp_snd::init(){
    con.reset(new abmt::io::udp(lst,{param_target, param_port}));
    lst.wait(0);
}

void udp_snd::tick(){
    lst.wait(0); // calls on_new_data and other events
    if(in.size() > 0){
        con->send(in.c_str(), in.size());
    }
}


void udp_rcv::init(){
    con.reset(new abmt::io::udp(lst,{}, {param_target, param_port}));
    con->on_new_data = [this](abmt::blob& b)->size_t{
		out += b.str();
	    return b.size;
	};
	lst.wait(0);
}

bool udp_rcv::poll(){
    out = "";
    lst.wait(1000); // calls on_new_data and other events
    if(out != ""){
        return true;
    }
    return false;
}

void tcp_server::init(){
    server.reset(new abmt::io::tcp_server(lst,{param_target, param_port}));
    server->on_new_connection = [&](abmt::io::tcp_ptr p){
		p->on_close = [this](){
		    for (auto it = connections.begin() ; it != connections.end(); ) {
                if ( (*it)->connected == false ) {
                    it = connections.erase(it);
                } else {
                    ++it;
                }
            }
		};
		p->on_new_data = [this](abmt::blob& b)->size_t{
			out += b.str();
		    return b.size;
		};
		connections.push_back(p);
	};
}

void tcp_server::tick(){
    out = "";
    lst.wait(0); // calls on_new_data and other events
    if(in_snd_to_all.size() == 0){
        return;
    }
    for(auto con:connections){
        if(con->connected){
            con->send(in_snd_to_all.c_str(), in_snd_to_all.size());
        }
    }
}



void tcp_client::init(){
    con = abmt::io::tcp::connect_to(lst,{param_target,param_port});
    con->on_new_data = [this](abmt::blob& b)->size_t{
		out += b.str();
		return b.size;
	};
}

void tcp_client::tick(){
    out = "";
    lst.wait(0); // calls on_new_data and other events
    if(con->connected && in.size() > 0){
        con->send(in.c_str(), in.size());
    }
}
