#include "playback.h"

#include <abmt/util/str_utils.h>
#include <sys/stat.h>
#include <sys/mman.h> 
#include <fcntl.h>
#include <unistd.h>
#include <abmt/blob.h>
#include <abmt/time.h>
#include <cstdio>
#include <cstdint>
#include <iostream>

using namespace std;
using namespace record_playback;
using namespace abmt::serialize;

void playback_base::open_file(std::string file_name){
    // open file
    fd = open(file_name.c_str(), O_RDONLY);
    abmt::os::die_if(fd < 0, "error open file " + file_name);
    struct stat s = {};
    fstat (fd, &s);
    file.size = s.st_size;
    file.data = (char*) mmap(0, file.size, PROT_READ, MAP_PRIVATE, fd, 0);
    abmt::os::die_if(file.data == MAP_FAILED || file.size == 0, "error map file " + file_name);
    abmt::os::die_if(file.str(0,5) != "arec1","Opend file " + file_name + " is not a recording.");
    
    start_time = file.get<int64_t>(5);
    // read header
    def_size = file.get<uint32_t>(5 + 8);
    def_ptr  = file.data + 5 + 8 + 4;
    decode_type = file.get<abmt::serialize::type_id>(5 + 8 + 4);
    
    
    offset_first_sample   = 5 + 8 + 4 + def_size;
    current_sample_offset = offset_first_sample;
}

bool playback_base::goto_next_sample(){
    bool found_sample = false;
    while(current_sample_offset < file.size){
        abmt::time sample_time(file.get<int64_t>(current_sample_offset));
        if( (sample_time + rec_delta_t) > abmt::time::now()){
            break; // next sample in future
        }
        found_sample = true;
        current_sample_time = sample_time;
        current_sample_size = file.get<uint32_t>(current_sample_offset + 8);;
        if(type_id_is_binary(decode_type)){
            current_sample_ptr = file.data + current_sample_offset + 8;
        }else{
            current_sample_ptr = file.data + current_sample_offset + 8 + 4;
        }
        current_sample_offset = current_sample_offset + 8 + 4 + current_sample_size;

    }
    return found_sample;
}

void playback_base::close_file(){
    munmap(file.data, file.size);
    close(fd);
}

void playback_sync::init(){
    auto dir = abmt::util::str_replace_date(param_dir);
    auto file_name = abmt::util::str_replace_date(param_filename);
    open_file(dir + "/" + file_name);
    if(param_start_now){
        rec_delta_t = abmt::time::now() - start_time;
    }
    out.def.data = def_ptr;
    out.def.size = def_size;
    out.data.data = 0;
    out.data.size = 0;
}

void playback_sync::tick(){
    if(goto_next_sample()){
       out.data.data = current_sample_ptr;
       out.data.size = current_sample_size;
    }
}

void playback_sync::final(){
    close_file();
}

void playback_async::init(){
    std::string dir = abmt::util::str_replace_date(param_dir);
    std::string file_name = abmt::util::str_replace_date(param_filename);
    open_file(dir + "/" + file_name);
    if(param_start_now){
        rec_delta_t = abmt::time::now() - start_time;
    }
    out.def.data = def_ptr;
    out.def.size = def_size;
    out.data.data = 0;
    out.data.size = 0;
}

bool playback_async::poll(){
    if(goto_next_sample()){
       out.data.data = current_sample_ptr;
       out.data.size = current_sample_size;
       return true;
    }
    return false;
}

void playback_async::final(){
    close_file();
}

