#ifndef RECORD_PLAYBACK_PLAYBACK_H
#define RECORD_PLAYBACK_PLAYBACK_H RECORD_PLAYBACK_PLAYBACK_H

#include <string>
#include <abmt/serialize.h>
#include <abmt/time.h>
#include <iostream>

namespace record_playback{
    
struct playback_base{
    int        fd = 0;
    abmt::blob file;
    uint32_t   def_size;
    char*      def_ptr;
    uint32_t   offset_first_sample;
    abmt::serialize::type_id decode_type;
    
    abmt::time start_time;
    abmt::time rec_delta_t = 0;
    
    size_t     current_sample_offset = 0;
    size_t     current_sample_size = 0;
    char*      current_sample_ptr  = 0;
    abmt::time current_sample_time = 0;
    
    playback_base(const playback_base&) = delete; //ptr ownership
    void open_file(std::string);
    /// returns false, when no next sample was found
    /// next sample may be in future or eof.
    bool goto_next_sample();
    void close_file();
};

//@node: auto
//@raster: auto
struct playback_sync: playback_base{
    std::string param_filename = "record.arec";
    std::string param_dir = "../rec/#Y#M#D_#h#m";
    bool        param_start_now = true;
    
    abmt::serialize::decoder out;
    
    void init();
    void tick();
    void final();
};

//@node: auto
//@raster: source
struct playback_async: playback_base{
    std::string param_filename = "record.arec";
    std::string param_dir = "../rec/#Y#M#D_#h#m";
    bool        param_start_now = true;
    
    abmt::serialize::decoder out;
    
    void init();
    bool poll();
    void final();
    
};
    

} // namespace record_playback

#endif // RECORD_PLAYBACK_PLAYBACK_H
