#include "record.h"

#include <sys/stat.h>
#include <sys/mman.h> 
#include <fcntl.h>
#include <unistd.h>
#include <abmt/blob.h>
#include <abmt/os.h>
#include <abmt/time.h>
#include <abmt/mutex.h>
#include <abmt/util/str_utils.h>
#include <cstdio>
#include <cstdint>
#include <iostream>
#include <filesystem>


using namespace std;
using namespace record_playback;
using namespace abmt::serialize;

abmt::mutex start_time_mutex;

abmt::time record_playback::get_global_start_time(){
    static abmt::time global_start_time = 0;
    start_time_mutex.lock();
    if(global_start_time == 0){
        global_start_time = abmt::time::now();
    }
    start_time_mutex.unlock();
    return global_start_time;
}

/**
 *  arec1 fileformat:
 *  ds = def_size
 *  ss = sample_size
 * 
 *  +---------+------------+----------+---------+-------------+-------------+------------+-----+
 *  | "arec1" | start_time | def_size | def     | sample_time | sample_size | sample     | ... |
 *  +---------+------------+----------+---------+-------------+-------------+------------+-----+
 *  |  0 (5)  |   5 (8)    |  13 (4)  | 17 (ds) |  17+ds (8)  |  25+ds (4)  | 29+ds (ss) | ... |
 *  +---------+------------+----------+---------+-------------+-------------+------------+-----+
 *  |  char   |  int64_t   | uint32_t | binary  |   int64_t   |  uint32_t   |  binary    | ... |
 *  +---------+------------+----------+---------+-------------+-------------+------------+-----+
 *  
 *  All types are little endian.
 * 
 */

void record::init(){
    start_time = get_global_start_time();
    auto dir = abmt::util::str_replace_date(param_dir,start_time);
    std::filesystem::create_directories(dir);
    auto file_name = abmt::util::str_replace_date(param_filename,start_time);
    auto path = dir + "/" + file_name;
    file = fopen(path.c_str(), "wb");
    abmt::os::die_if(file == 0, "record: unable to open " + path);
    abmt::os::log("create: " + path); // copy and paste me
    fwrite("arec1",5,1,(FILE*)file);
    int64_t rec_start_time = start_time.ns_since_1970;
    fwrite(&rec_start_time,8,1,(FILE*)file);
    uint32_t def_size = in.get_def_size();
    fwrite(&def_size,4,1,(FILE*)file);
    abmt::blob_shared def(def_size);
    in.get_def(def.data, def_size);
    fwrite(def.data,def_size,1,(FILE*)file);
}

void record::tick(){
    int64_t now = abmt::time::now().ns_since_1970;
    fwrite(&now,8,1,(FILE*)file);
    uint32_t size = in.get_size();
    if( type_id_is_binary(in.id) == false ){
        // is_binarydecoder types have size value at the beginning
        fwrite(&size,4,1,(FILE*)file);
    }
    abmt::blob_shared ser_data(size);
    in.serialize(ser_data.data, size);
    fwrite(ser_data.data,size,1,(FILE*)file);
}

void record::final(){
    fclose((FILE*)file);
}

