#include "basic_3d.h"
#include <iostream>

using namespace std;
using namespace scene3d;

void move::tick(){
    out_pose.x = in_pos.x;
    out_pose.y = in_pos.y;
    out_pose.z = in_pos.z;
}

void rotate::tick(){
    out_pose = abmt::rot_vec(in_vec, in_ang);
}

void cube::init(){
   cube.reset(new abmt::cuboid);
   out_sc.cubes.push_back(cube);
}

void cube::tick(){
    cube->size = in_size;
    cube->color = in_color;
}

void point3d::init(){
   point.reset(new abmt::point3d);
   out_sc.points.push_back(point);
}

void point3d::tick(){
    point->size = in_size;
    point->color = in_color;
}

void line3d::init(){
   line.reset(new abmt::line3d);
   out_sc.lines.push_back(line);
}

void line3d::tick(){
    line->p1 = in_p1;
    line->p2 = in_p2;
    line->color = in_color;
}

void model3d::init(){
   model.reset(new abmt::model);
   model->path = param_file_name;
   model->p[0][0] = param_scale;
   model->p[1][1] = param_scale;
   model->p[2][2] = param_scale;
   out_sc.models.push_back(model);
}

void spline3d::init(){
   spline.reset(new abmt::spline);
   spline->scale = param_scale;
   out_sc.splines.push_back(spline);
}

void spline3d::tick(){
    spline->img = in_img;
}

void grid_xy::init(){
    for(int x = -param_steps_x/2; x <= param_steps_x/2; ++x){
        auto l = std::make_shared<abmt::line3d>();
        l->color = {85,85,85,255};
        l->p1 = {x*param_step_size, -param_steps_y/2*param_step_size, param_z};
        l->p2 = {x*param_step_size,  param_steps_y/2*param_step_size, param_z};
        out_sc.lines.push_back(l);      
    }
    
    for(int y = -param_steps_y/2; y <= param_steps_y/2; ++y){
        auto l = std::make_shared<abmt::line3d>();
        l->color = {85,85,85,255};
        l->p1 = {-param_steps_x/2*param_step_size, y*param_step_size, param_z};
        l->p2 = { param_steps_x/2*param_step_size, y*param_step_size, param_z};
        out_sc.lines.push_back(l);      
    }

}

void grid_xz::init(){
    for(int x = -param_steps_x/2; x <= param_steps_x/2; ++x){
        auto l = std::make_shared<abmt::line3d>();
        l->color = {85,85,85,255};
        l->p1 = {x*param_step_size, param_y, -param_steps_z/2*param_step_size};
        l->p2 = {x*param_step_size, param_y, param_steps_z/2*param_step_size};
        out_sc.lines.push_back(l);      
    }
    
    for(int z = -param_steps_z/2; z <= param_steps_z/2; ++z){
        auto l = std::make_shared<abmt::line3d>();
        l->color = {85,85,85,255};
        l->p1 = {-param_steps_x/2*param_step_size, param_y, z*param_step_size};
        l->p2 = { param_steps_x/2*param_step_size, param_y, z*param_step_size};
        out_sc.lines.push_back(l);      
    }

}

void grid_yz::init(){
    for(int y = -param_steps_y/2; y <= param_steps_y/2; ++y){
        auto l = std::make_shared<abmt::line3d>();
        l->color = {85,85,85,255};
        l->p1 = {param_x, y*param_step_size, -param_steps_z/2*param_step_size};
        l->p2 = {param_x, y*param_step_size,  param_steps_z/2*param_step_size};
        out_sc.lines.push_back(l);      
    }
    for(int z = -param_steps_z/2; z <= param_steps_z/2; ++z){
        auto l = std::make_shared<abmt::line3d>();
        l->color = {85,85,85,255};
        l->p1 = {param_x, -param_steps_y/2*param_step_size, z*param_step_size};
        l->p2 = {param_x,  param_steps_y/2*param_step_size, z*param_step_size};
        out_sc.lines.push_back(l);      
    }
}

void axis::init(){
  auto lr = std::make_shared<abmt::line3d>();
  lr->color = {255,0,0,255};
  lr->p2.x = param_len;
  out_sc.lines.push_back(lr);
  
  auto lg = std::make_shared<abmt::line3d>();
  lg->color = {0,255,0,255};
  lg->p2.y = param_len;
  out_sc.lines.push_back(lg);
 
  auto lb = std::make_shared<abmt::line3d>();
  lb->color = {0,0,255,255};
  lb->p2.z = param_len;
  out_sc.lines.push_back(lb);
}