#ifndef SERIALIZATION_SERIALIZATION_EXAMPLE_H
#define SERIALIZATION_SERIALIZATION_EXAMPLE_H SERIALIZATION_SERIALIZATION_EXAMPLE_H

#include <cstdint>
#include <string>
#include <abmt/json.h>
#include <abmt/serialize.h>
#include <iostream>

namespace serialization{

struct child_obj{
    double a = 1;
    std::string str = "str";
    
    abmt::serialize::type define_type(){
        static const abmt::serialize::member list[] = {
                        ABMT_DEF_MEMBER(a),
                        ABMT_DEF_MEMBER(str),
        };
        return list;
    }

};

struct child_json{
    double c = 2;
    double d = 3;
    
    child_json(){
        static double cnt = 0;
        cnt++;
        c = cnt;
    }
    
    std::string to_json_str(){
        abmt::json j;
        j["c"] = c;
        j["d"] = d;
        return j.dump();
    }
    
    void from_json_str(std::string s){
        auto j = abmt::json::parse(s);
        c = j["c"];
        d = j["d"];
    }
};

struct child_str{
    std::string name = "default name";
    
    std::string to_string(){
        return name;
    }
    
    void from_string(std::string s){
        name = s;
    }
};

/// example for an type definition outside of the class
/// child_ext could be in an other external header
/// The type definition is at the bottom of this file
/// outside of the namespace!
struct child_ext1{
    int i = 4;
};
struct child_ext2{
    int32_t i = 5;
    child_ext2(){
        static int32_t cnt = 5;
        cnt++;
        i = cnt;
    }
};


struct example{
    child_obj o;
    child_json j;
    child_str s;
    child_ext1 e1;
    child_ext2 e2;
    int i = 6;
    
    abmt::serialize::type define_type(){
        static const abmt::serialize::member list[] = {
                        ABMT_DEF_MEMBER(o),
                        ABMT_DEF_MEMBER(j),
                        ABMT_DEF_MEMBER(s),
                        ABMT_DEF_MEMBER(e1),
                        ABMT_DEF_MEMBER(e2),
                        ABMT_DEF_MEMBER(i),
        };
        return list;
    }
    
    void print_e2(){
        std::cout << "e2.i: " << e2.i << std::endl;
    }
    
};

} // namespace serialization

template<> abmt::serialize::type abmt::serialize::define_type(serialization::child_ext1* target){
	static const abmt::serialize::member map[] = {
			{"i", target->i}

	 };
	return map;
}

template<> abmt::serialize::type abmt::serialize::define_type(serialization::child_ext2* target){
	return blob_type({
		/* get_size    */
		[](void* obj) -> size_t { return sizeof(int32_t); },
		/* obj_to_data */  
		[](void* obj, void* data, size_t size) {
		    auto o = (serialization::child_ext2*) obj;
		    memcpy(data, &o->i,  size); 
		},
		/* data_to_obj */
		[](void* obj, void* data, size_t size) {
		    auto o = (serialization::child_ext2*) obj;
		    memcpy(&o->i,  data, size); 
		}
	});
}

#endif // SERIALIZATION_SERIALIZATION_EXAMPLE_H
