/**
 * Author: Hendrik van Arragon, 2022
 * SPDX-License-Identifier: MIT
 */

#include "model_com_eio.h"
#include <abmt/version.h>
#include <iostream>
#include <cstdlib>

using namespace std;

model_com_eio::model_com_eio(abmt::io::event_list& lst):e(lst),timer(e,1000){
	connect();
	timer.on_expiraton = [this]{
		if(connected == true){
			adapter.send_model_online();
		}else{
			connect();
		}
	};
	adapter.on_version_error = [this]{
		static int output_counter = 0;
		if(output_counter == 0){
			cout << "Error connecting to ABMT. Wrong version. Expected version " << ABMT_VERSION << endl;
			output_counter = 20;
		}else{
			output_counter--;
		}
		tcp_con->close();
		tcp_con->on_open = []{}; // temporary bugfix becaue on_open is delayed...
	};
}

void model_com_eio::connect(){
	connected = false;
	uint16_t port = 15101;
	const char* port_str = std::getenv("ABMT_MODEL_PORT");
	if(port_str != nullptr){
		port = std::stoul(port_str);
	}
	tcp_con = abmt::io::tcp::connect_to(e,{"127.0.0.1",port});
	tcp_con->on_open = [this]{
		connected = true;
	};

	adapter.connection.set_sink(tcp_con);
	adapter.connection.set_source(tcp_con);

	// override on_close set by set_connection
	tcp_con->on_close = [this]{
		connected = false;
		adapter.connection.set_sink(abmt::io::sink());
		adapter.connection.set_source(abmt::io::source());
	};



}

void model_com_eio::set_model(abmt::rt::model* m){
	mdl = m;
	adapter.set_model(mdl);
}
