/**
 * Author: Hendrik van Arragon, 2022
 * SPDX-License-Identifier: MIT
 */

#include "time_from_rec.h"

#include <sys/stat.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <unistd.h>
#include <cstdio>
#include <fstream>
#include <abmt/os.h>
#include <abmt/blob.h>
#include <abmt/util/str_utils.h>

using namespace std;

abmt::time get_start_time_from_arec(std::string file_name){
	abmt::time res;
	char arec1[5];
	auto file = fopen(file_name.c_str(),"rb");
	auto bytes_read = fread(arec1,5,1,file);
	abmt::os::die_if(bytes_read != 1 && std::string(arec1) != "arec1",
			"File '"+file_name+"' is not a arec-file"
	);
	int64_t time;
	bytes_read = fread(&time,8,1,file);
	abmt::os::die_if(bytes_read != 1, "Unable to read start time: " + file_name);
	res.ns_since_1970 = time;
	return res;
}

abmt::time get_end_time_from_arec(std::string file_name){
	abmt::time res;
	auto fd = open(file_name.c_str(), O_RDONLY);
	abmt::os::die_if(fd < 0, "error open file " + file_name);
	struct stat s = {};
	fstat (fd, &s);
	abmt::blob file;
	file.size = s.st_size;
	file.data = (char*) mmap(0, file.size, PROT_READ, MAP_PRIVATE, fd, 0);
	abmt::os::die_if(file.data == MAP_FAILED || file.size == 0, "error map file " + file_name);
	abmt::os::die_if(file.str(0,5) != "arec1","Opend file " + file_name + " is not a recording.");

	res.ns_since_1970 = file.get<int64_t>(5);
	auto def_size = file.get<uint32_t>(5 + 8);
	size_t current_sample_offset = 5 + 8 + 4 + def_size;;

	while(current_sample_offset < file.size){
		res.ns_since_1970 = file.get<int64_t>(current_sample_offset);
		auto current_sample_size = file.get<uint32_t>(current_sample_offset + 8);;
		current_sample_offset = current_sample_offset + 8 + 4 + current_sample_size;
	}

	munmap(file.data, file.size);
	close(fd);

	return res;
}

abmt::time get_start_time_from_csv(std::string file_name){
	abmt::date res;
	std::fstream file;
	file.open(file_name);
	abmt::os::die_if(file.is_open() == false, "Unable to open csv file: " +file_name);
	std::string line;
	std::getline(file,line); // header
	std::getline(file,line); // first line
	auto splitted = abmt::util::str_split_and_trim(line, ",");
	res = abmt::util::str_read_date(splitted[0]);
	file.close();

	if(res.is_zero()){
		return 0;
	}
	return res;
}

abmt::time get_end_time_from_csv(std::string file_name){
	abmt::date res;
	std::fstream file;
	file.open(file_name);
	abmt::os::die_if(file.is_open() == false, "unable to open csv file: " +file_name);
	std::string line;
	std::getline(file,line); // header
	while(std::getline(file,line)) {
        auto splitted = abmt::util::str_split_and_trim(line, ",");
        res = abmt::util::str_read_date(splitted[0]);
	}
	file.close();

	if(res.is_zero()){
		return 0;
	}
	return res;
}
