/**
 * Author: Hendrik van Arragon, 2021
 * SPDX-License-Identifier: MIT
 */

#ifndef UTIL_INC_BUFFER_H_
#define UTIL_INC_BUFFER_H_

#include <abmt/mutex.h>
#include <abmt/serialize.h>
#include <functional>

namespace abmt{

template<typename T, typename MUTEX = abmt::mutex>
class shared_data{
public:
	T data;
	MUTEX lock;
	shared_data(T initial_value){
		data = initial_value;
	}

	T get(){
		lock.lock();
		T copy = data;
		lock.unlock();
	   return copy;
	}

	void set(T new_value){
		lock.lock();;
		data = new_value;
		lock.unlock();
	}
};



template<typename T>
class buffer{
public:
	T data ;
	mutex lock;


	buffer(T d): data(d){

	}

	buffer(){
		data = 0;
	}

	operator T(){
		auto m = lock.get_scope_lock();
		return data;
	}

	T& operator=(T& rhs){
		lock.lock();
		data = rhs;
		lock.unlock();
		return rhs;
	}
};


// In ersten tests ist db (double buffer) halb so langsam wie buffer
// Es wurde nur mit kleinen Strukturen und etwas größeren
// Strukturen getestet (1MB). Nur bei 10k war db geringfügig schneller

template<typename T>
class db{
public:
	// double buffer;
	T data1 ;
	T data2;
	bool first = true;

	mutex write_lock;
	mutex read_lock;


	db(T d): data1(d),data2(d){

	}

	db(){
		data1 = 0;
		data2 = 0;
	}

	operator T(){
		auto m = read_lock.get_scope_lock();
		if(first){
			return data1;
		}else{
			return data2;
		}
	}

	T& operator=(T& rhs){
		//cout << "operator" << endl;
		write_lock.lock();
		if(first){
			data2 = rhs;
		}else{
			data1 = rhs;
		}
		read_lock.lock();
		first = !first;
		read_lock.unlock();

		write_lock.unlock();

		return rhs;
	}
};

} // namespace abmt



#endif /* UTIL_INC_BUFFER_H_ */
