/**
 * Author: Hendrik van Arragon, 2021
 * SPDX-License-Identifier: MIT
 */

#ifndef UTIL_INC_CLOCK_H_
#define UTIL_INC_CLOCK_H_

#include <cstdint>
#include <string>

namespace abmt{

class time{
public:

	volatile int64_t ns_since_1970;

	time(int64_t ns=0);

	/// must be defined in runtime.
	static time now();

	static time ns(int64_t t);
	static time us(int64_t t);
	static time ms(int64_t t);
	static time sec(int64_t t);
	static time min(int64_t t);
	static time hour(int64_t t);
	static time day(int64_t t);
	static time week(int64_t t);
	static time year(int64_t t);

	double ns();
	double us();
	double ms();
	double sec();
	double min();
	double hour();
	double week();
	double year();

	time delta_to_now();

	bool operator==(const time &t);
	bool operator!=(const time &t);
	bool operator< (const time &t);
	bool operator> (const time &t);
	bool operator<=(const time &t);
	bool operator>=(const time &t);

	time& operator+=(const time &t);
	time& operator-=(const time &t);
	time& operator*=(const time &t);

	time& operator/=(const time &t);
	time operator+(const time &t) const;
	time operator-(const time &t) const;
	time operator*(const time &t) const;
	time operator/(const time &t) const;
	time operator*(const double& n) const;
	time operator/(const double& n);

};

// for things like time_obj*1.5
time operator*(const double& n,const time &rhs);

struct date{
	size_t year;
	size_t month;
	size_t day;
	size_t hour;
	size_t min;
	float  sec;

	date(time t = abmt::time::now());
	static date zero();
	operator time();
	bool is_zero();
};

} // namespace

#endif /* UTIL_INC_CLOCK_H_ */
