/**
 * Author: Hendrik van Arragon, 2021
 * SPDX-License-Identifier: MIT
 */

#ifndef SHARED_ABMT_PIMPL_H_
#define SHARED_ABMT_PIMPL_H_

#include <functional>
#include <typeinfo>
#include <stdexcept>

namespace abmt{
namespace util{


struct pimpl{
	void* p = 0;
	std::function<void()> del = []{};
	std::size_t tid = 0;

	pimpl() = default;

	template<typename T>
	pimpl(T* instance){
		set(instance);
	}

	pimpl(const pimpl& p) = delete;

	template<typename T>
	void set(T* instance){
		if(p != 0){
			del();
		}
		del = [this]{
			delete (T*)p;
		};
		p = instance;
		tid = typeid(T).hash_code();
	}

	template<typename T>
	T& get(){
		if(p == 0){
			throw std::runtime_error("No object was set to pimpl!");
		}else if(tid != typeid(T).hash_code()){
			throw std::runtime_error("The type that is requested is different than the type that was set to pimpl!");
		}
		return *(T*)p;
	}

	/// when you know what you are doing and what your past me has done... (returns cast without type-check)
	template<typename T>
	T& get2(){
		return *(T*)p;
	}


	~pimpl(){
		if(p != 0){
			del();
		}
	}
};

} //namespace util
} //namespace abmt

#endif /* SHARED_ABMT_PIMPL_H_ */
